# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-20 11:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leaflet_storage', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='datalayer',
            name='rank',
            field=models.SmallIntegerField(default=0),
        ),
    ]
