# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-10 06:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leaflet_storage', '0002_auto_20160520_1114'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='datalayer',
            options={'ordering': ('rank',)},
        ),
        migrations.AlterField(
            model_name='pictogram',
            name='pictogram',
            field=models.ImageField(upload_to='pictogram'),
        ),
    ]
