{% load leaflet_storage_tags %}
    <div id="{{ prefix }}{{ map.pk }}" class="map_fragment"></div>
    <script type="text/javascript">
        new L.Storage.Map("{{ prefix }}{{ map.pk }}", {{ map_settings|notag|safe }});
    </script>
