<!DOCTYPE html>
<html lang="en">
<head>
    <title>{% block title %}Leaflet Storage Map{% endblock %}</title>
    {% block extra_head %}
    {% endblock %}
</head>
<body {% block body_id %}{% endblock %} {% block body_class %}{% endblock %}>
    {% block extrabody %}
    {% endblock %}
    {% block body %}
        <div id="header">
            {% block header %}
            {% endblock %}
        </div>
        <div id="sidebar">
            {% block sidebar %}
            {% endblock %}
        </div>
        <div id="content">
            {% block content %}
            {% endblock %}
        </div>
        <div id="footer">
            {% block footer %}
            {% endblock %}
        </div>
    {% endblock %}
</body>
</html>
